import os, sys
from distutils.msvccompiler import get_build_version


def get_config():
    from setup_common import get_metadata_and_options, enabled, create_release_file

    metadata, options = get_metadata_and_options()

    connector = options["connector"]

    extra_objects = []

    # client = "mysqlclient"
    client = "mariadbclient"

    vcversion = int(get_build_version())
    if client == "mariadbclient":
        library_dirs = [os.path.join(connector, 'lib', 'mariadb')]
        libraries = ['kernel32', 'advapi32', 'wsock32', 'shlwapi', 'Ws2_32', client ]
        include_dirs = [os.path.join(connector, 'include', 'mariadb')]
    else:
        library_dirs = [os.path.join(connector, r'lib\vs%d' % vcversion),
                        os.path.join(connector, "lib")]
        libraries = ['kernel32', 'advapi32', 'wsock32', client ]
        include_dirs = [os.path.join(connector, r'include')]

    extra_compile_args = ['/Zl', '/D_CRT_SECURE_NO_WARNINGS' ]
    extra_link_args = ['/MANIFEST']

    name = "mysqlclient"
    metadata['name'] = name

    define_macros = [
        ('version_info', metadata['version_info']),
        ('__version__', metadata['version']),
        ]
    create_release_file(metadata)
    del metadata['version_info']
    ext_options = dict(
        library_dirs = library_dirs,
        libraries = libraries,
        extra_compile_args = extra_compile_args,
        extra_link_args = extra_link_args,
        include_dirs = include_dirs,
        extra_objects = extra_objects,
        define_macros = define_macros,
    )
    return metadata, ext_options

if __name__ == "__main__":
    sys.stderr.write("""You shouldn't be running this directly; it is used by setup.py.""")



import unittest

from MySQLdb import _mysql
import MySQLdb
from MySQLdb.constants import FIELD_TYPE
from configdb import connection_factory
import warnings
warnings.simplefilter("ignore")


class TestDBAPISet(unittest.TestCase):
    def test_set_equality(self):
        self.assertTrue(MySQLdb.STRING == MySQLdb.STRING)

    def test_set_inequality(self):
        self.assertTrue(MySQLdb.STRING != MySQLdb.NUMBER)

    def test_set_equality_membership(self):
        self.assertTrue(FIELD_TYPE.VAR_STRING == MySQLdb.STRING)

    def test_set_inequality_membership(self):
        self.assertTrue(FIELD_TYPE.DATE != MySQLdb.STRING)


class TestCoreModule(unittest.TestCase):
    """Core _mysql module features."""

    def test_version(self):
        """Version information sanity."""
        self.assertTrue(isinstance(_mysql.__version__, str))

        self.assertTrue(isinstance(_mysql.version_info, tuple))
        self.assertEqual(len(_mysql.version_info), 5)

    def test_client_info(self):
        self.assertTrue(isinstance(_mysql.get_client_info(), str))

    def test_escape_string(self):
        self.assertEqual(_mysql.escape_string(b'foo"bar'),
                         b'foo\\"bar', "escape byte string")
        self.assertEqual(_mysql.escape_string(u'foo"bar'),
                         b'foo\\"bar', "escape unicode string")


class CoreAPI(unittest.TestCase):
    """Test _mysql interaction internals."""

    def setUp(self):
        self.conn = connection_factory(use_unicode=True)

    def tearDown(self):
        self.conn.close()

    def test_thread_id(self):
        tid = self.conn.thread_id()
        self.assertTrue(isinstance(tid, int),
                        "thread_id didn't return an int.")

        self.assertRaises(TypeError, self.conn.thread_id, ('evil',),
                          "thread_id shouldn't accept arguments.")

    def test_affected_rows(self):
        self.assertEqual(self.conn.affected_rows(), 0,
                         "Should return 0 before we do anything.")


    #def test_debug(self):
        ## FIXME Only actually tests if you lack SUPER
        #self.assertRaises(MySQLdb.OperationalError,
                          #self.conn.dump_debug_info)

    def test_charset_name(self):
        self.assertTrue(isinstance(self.conn.character_set_name(), str),
                        "Should return a string.")

    def test_host_info(self):
        self.assertTrue(isinstance(self.conn.get_host_info(), str),
                        "Should return a string.")

    def test_proto_info(self):
        self.assertTrue(isinstance(self.conn.get_proto_info(), int),
                        "Should return an int.")

    def test_server_info(self):
        self.assertTrue(isinstance(self.conn.get_server_info(), str),
                        "Should return a string.")

    def test_client_flag(self):
        conn = connection_factory(
            use_unicode=True,
            client_flag=MySQLdb.constants.CLIENT.FOUND_ROWS)

        self.assertIsInstance(conn.client_flag, (int, MySQLdb.compat.long))
        self.assertTrue(conn.client_flag & MySQLdb.constants.CLIENT.FOUND_ROWS)
        with self.assertRaises(TypeError if MySQLdb.compat.PY2 else AttributeError):
            conn.client_flag = 0

        conn.close()

    def test_fileno(self):
        self.assertGreaterEqual(self.conn.fileno(), 0)



try:
    # Python 2.x
    from ConfigParser import SafeConfigParser
except ImportError:
    # Python 3.x
    from configparser import ConfigParser as SafeConfigParser

def get_metadata_and_options():
    config = SafeConfigParser()
    config.read(['metadata.cfg', 'site.cfg'])

    metadata = dict(config.items('metadata'))
    options = dict(config.items('options'))

    metadata['py_modules'] = list(filter(None, metadata['py_modules'].split('\n')))
    metadata['classifiers'] = list(filter(None, metadata['classifiers'].split('\n')))

    return metadata, options

def enabled(options, option):
    value = options[option]
    s = value.lower()
    if s in ('yes','true','1','y'):
        return True
    elif s in ('no', 'false', '0', 'n'):
        return False
    else:
        raise ValueError("Unknown value %s for option %s" % (value, option))

def create_release_file(metadata):
    rel = open("MySQLdb/release.py",'w')
    rel.write("""
__author__ = "%(author)s <%(author_email)s>"
version_info = %(version_info)s
__version__ = "%(version)s"
""" % metadata)
    rel.close()



"""Configure database connection for tests."""

from os import environ, path

tests_path = path.dirname(__file__)
conf_file = environ.get('TESTDB', 'default.cnf')
conf_path = path.join(tests_path, conf_file)
connect_kwargs = dict(
    read_default_file = conf_path,
    read_default_group = "MySQLdb-tests",
)


def connection_kwargs(kwargs):
    db_kwargs = connect_kwargs.copy()
    db_kwargs.update(kwargs)
    return db_kwargs


def connection_factory(**kwargs):
    import MySQLdb
    db_kwargs = connection_kwargs(kwargs)
    db = MySQLdb.connect(**db_kwargs)
    return db



from __future__ import print_function, absolute_import

import pytest
import MySQLdb.cursors
from configdb import connection_factory


_conns = []
_tables = []

def connect(**kwargs):
    conn = connection_factory(**kwargs)
    _conns.append(conn)
    return conn


def teardown_function(function):
    if _tables:
        c = _conns[0]
        cur = c.cursor()
        for t in _tables:
            cur.execute("DROP TABLE %s" % (t,))
        cur.close()
        del _tables[:]

    for c in _conns:
        c.close()
    del _conns[:]


def test_executemany():
    conn = connect()
    cursor = conn.cursor()

    cursor.execute("create table test (data varchar(10))")
    _tables.append("test")

    m = MySQLdb.cursors.RE_INSERT_VALUES.match("INSERT INTO TEST (ID, NAME) VALUES (%s, %s)")
    assert m is not None, 'error parse %s'
    assert m.group(3) == '', 'group 3 not blank, bug in RE_INSERT_VALUES?'

    m = MySQLdb.cursors.RE_INSERT_VALUES.match("INSERT INTO TEST (ID, NAME) VALUES (%(id)s, %(name)s)")
    assert m is not None, 'error parse %(name)s'
    assert m.group(3) == '', 'group 3 not blank, bug in RE_INSERT_VALUES?'

    m = MySQLdb.cursors.RE_INSERT_VALUES.match("INSERT INTO TEST (ID, NAME) VALUES (%(id_name)s, %(name)s)")
    assert m is not None, 'error parse %(id_name)s'
    assert m.group(3) == '', 'group 3 not blank, bug in RE_INSERT_VALUES?'

    m = MySQLdb.cursors.RE_INSERT_VALUES.match("INSERT INTO TEST (ID, NAME) VALUES (%(id_name)s, %(name)s) ON duplicate update")
    assert m is not None, 'error parse %(id_name)s'
    assert m.group(3) == ' ON duplicate update', 'group 3 not ON duplicate update, bug in RE_INSERT_VALUES?'

    # https://github.com/PyMySQL/mysqlclient-python/issues/178
    m = MySQLdb.cursors.RE_INSERT_VALUES.match("INSERT INTO bloup(foo, bar)VALUES(%s, %s)")
    assert m is not None

    # cursor._executed myst bee "insert into test (data) values (0),(1),(2),(3),(4),(5),(6),(7),(8),(9)"
    # list args
    data = range(10)
    cursor.executemany("insert into test (data) values (%s)", data)
    assert cursor._executed.endswith(b",(7),(8),(9)"), 'execute many with %s not in one query'

    # dict args
    data_dict = [{'data': i} for i in range(10)]
    cursor.executemany("insert into test (data) values (%(data)s)", data_dict)
    assert cursor._executed.endswith(b",(7),(8),(9)"), 'execute many with %(data)s not in one query'

    # %% in column set
    cursor.execute("""\
        CREATE TABLE percent_test (
            `A%` INTEGER,
            `B%` INTEGER)""")
    try:
        q = "INSERT INTO percent_test (`A%%`, `B%%`) VALUES (%s, %s)"
        assert MySQLdb.cursors.RE_INSERT_VALUES.match(q) is not None
        cursor.executemany(q, [(3, 4), (5, 6)])
        assert cursor._executed.endswith(b"(3, 4),(5, 6)"), "executemany with %% not in one query"
    finally:
        cursor.execute("DROP TABLE IF EXISTS percent_test")


def test_pyparam():
    conn = connect()
    cursor = conn.cursor()

    cursor.execute(u"SELECT %(a)s, %(b)s", {u'a': 1, u'b': 2})
    assert cursor._executed == b"SELECT 1, 2"
    cursor.execute(b"SELECT %(a)s, %(b)s", {b'a': 3, b'b': 4})
    assert cursor._executed == b"SELECT 3, 4"



import mock
import unittest
from time import gmtime
from datetime import time, date, datetime, timedelta

from MySQLdb import times

import warnings
warnings.simplefilter("ignore")


class TestX_or_None(unittest.TestCase):
    def test_date_or_none(self):
        assert times.Date_or_None('1969-01-01') == date(1969, 1, 1)
        assert times.Date_or_None('2015-01-01') == date(2015, 1, 1)
        assert times.Date_or_None('2015-12-13') == date(2015, 12, 13)

        assert times.Date_or_None('') is None
        assert times.Date_or_None('fail') is None
        assert times.Date_or_None('2015-12') is None
        assert times.Date_or_None('2015-12-40') is None
        assert times.Date_or_None('0000-00-00') is None

    def test_time_or_none(self):
        assert times.Time_or_None('00:00:00') == time(0, 0)
        assert times.Time_or_None('01:02:03') == time(1, 2, 3)
        assert times.Time_or_None('01:02:03.123456') == time(1, 2, 3, 123456)

        assert times.Time_or_None('') is None
        assert times.Time_or_None('fail') is None
        assert times.Time_or_None('24:00:00') is None
        assert times.Time_or_None('01:02:03.123456789') is None

    def test_datetime_or_none(self):
        assert times.DateTime_or_None('1000-01-01') == date(1000, 1, 1)
        assert times.DateTime_or_None('2015-12-13') == date(2015, 12, 13)
        assert times.DateTime_or_None('2015-12-13 01:02') == datetime(2015, 12, 13, 1, 2)
        assert times.DateTime_or_None('2015-12-13T01:02') == datetime(2015, 12, 13, 1, 2)
        assert times.DateTime_or_None('2015-12-13 01:02:03') == datetime(2015, 12, 13, 1, 2, 3)
        assert times.DateTime_or_None('2015-12-13T01:02:03') == datetime(2015, 12, 13, 1, 2, 3)
        assert times.DateTime_or_None('2015-12-13 01:02:03.123') == datetime(2015, 12, 13, 1, 2, 3, 123000)
        assert times.DateTime_or_None('2015-12-13 01:02:03.000123') == datetime(2015, 12, 13, 1, 2, 3, 123)
        assert times.DateTime_or_None('2015-12-13 01:02:03.123456') == datetime(2015, 12, 13, 1, 2, 3, 123456)
        assert times.DateTime_or_None('2015-12-13T01:02:03.123456') == datetime(2015, 12, 13, 1, 2, 3, 123456)

        assert times.DateTime_or_None('') is None
        assert times.DateTime_or_None('fail') is None
        assert times.DateTime_or_None('0000-00-00 00:00:00') is None
        assert times.DateTime_or_None('0000-00-00 00:00:00.000000') is None
        assert times.DateTime_or_None('2015-12-13T01:02:03.123456789') is None

    def test_timedelta_or_none(self):
        assert times.TimeDelta_or_None('-1:0:0') == timedelta(0, -3600)
        assert times.TimeDelta_or_None('1:0:0') == timedelta(0, 3600)
        assert times.TimeDelta_or_None('12:55:30') == timedelta(0, 46530)
        assert times.TimeDelta_or_None('12:55:30.123456') == timedelta(0, 46530, 123456)
        assert times.TimeDelta_or_None('12:55:30.123456789') == timedelta(0, 46653, 456789)
        assert times.TimeDelta_or_None('12:55:30.123456789123456') == timedelta(1429, 37719, 123456)

        assert times.TimeDelta_or_None('') is None
        assert times.TimeDelta_or_None('0') is None
        assert times.TimeDelta_or_None('fail') is None


class TestTicks(unittest.TestCase):
    @mock.patch('MySQLdb.times.localtime', side_effect=gmtime)
    def test_date_from_ticks(self, mock):
        assert times.DateFromTicks(0) == date(1970, 1, 1)
        assert times.DateFromTicks(1430000000) == date(2015, 4, 25)

    @mock.patch('MySQLdb.times.localtime', side_effect=gmtime)
    def test_time_from_ticks(self, mock):
        assert times.TimeFromTicks(0) == time(0, 0, 0)
        assert times.TimeFromTicks(1431100000) == time(15, 46, 40)
        assert times.TimeFromTicks(1431100000.123) == time(15, 46, 40)

    @mock.patch('MySQLdb.times.localtime', side_effect=gmtime)
    def test_timestamp_from_ticks(self, mock):
        assert times.TimestampFromTicks(0) == datetime(1970, 1, 1, 0, 0, 0)
        assert times.TimestampFromTicks(1430000000) == datetime(2015, 4, 25, 22, 13, 20)
        assert times.TimestampFromTicks(1430000000.123) == datetime(2015, 4, 25, 22, 13, 20)


class TestToLiteral(unittest.TestCase):
    def test_datetime_to_literal(self):
        assert times.DateTime2literal(datetime(2015, 12, 13), '') == b"'2015-12-13 00:00:00'"
        assert times.DateTime2literal(datetime(2015, 12, 13, 11, 12, 13), '') == b"'2015-12-13 11:12:13'"
        assert times.DateTime2literal(datetime(2015, 12, 13, 11, 12, 13, 123456), '') == b"'2015-12-13 11:12:13.123456'"

    def test_datetimedelta_to_literal(self):
        d = datetime(2015, 12, 13, 1, 2, 3) - datetime(2015, 12, 13, 1, 2, 2)
        assert times.DateTimeDelta2literal(d, '') == b"'0 0:0:1'"


class TestFormat(unittest.TestCase):
    def test_format_timedelta(self):
        d = datetime(2015, 1, 1) - datetime(2015, 1, 1)
        assert times.format_TIMEDELTA(d) == '0 0:0:0'

        d = datetime(2015, 1, 1, 10, 11, 12) - datetime(2015, 1, 1, 8, 9, 10)
        assert times.format_TIMEDELTA(d) == '0 2:2:2'

        d = datetime(2015, 1, 1, 10, 11, 12) - datetime(2015, 1, 1, 11, 12, 13)
        assert times.format_TIMEDELTA(d) == '-1 22:58:59'

    def test_format_timestamp(self):
        assert times.format_TIMESTAMP(datetime(2015, 2, 3)) == '2015-02-03 00:00:00'
        assert times.format_TIMESTAMP(datetime(2015, 2, 3, 17, 18, 19)) == '2015-02-03 17:18:19'
        assert times.format_TIMESTAMP(datetime(15, 2, 3, 17, 18, 19)) == '0015-02-03 17:18:19'



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import capabilities
from datetime import timedelta
from contextlib import closing
import unittest
import MySQLdb
from MySQLdb.compat import unicode
from MySQLdb import cursors
from configdb import connection_factory
import warnings


warnings.filterwarnings('ignore')


class test_MySQLdb(capabilities.DatabaseTest):

    db_module = MySQLdb
    connect_args = ()
    connect_kwargs = dict(use_unicode=True, sql_mode="ANSI,STRICT_TRANS_TABLES,TRADITIONAL")
    create_table_extra = "ENGINE=INNODB CHARACTER SET UTF8"
    leak_test = False

    def quote_identifier(self, ident):
        return "`%s`" % ident

    def test_TIME(self):
        def generator(row,col):
            return timedelta(0, row*8000)
        self.check_data_integrity(
                 ('col1 TIME',),
                 generator)

    def test_TINYINT(self):
        # Number data
        def generator(row, col):
            v = (row*row) % 256
            if v > 127:
                v = v-256
            return v
        self.check_data_integrity(
            ('col1 TINYINT',),
            generator)

    def test_stored_procedures(self):
        db = self.connection
        c = self.cursor
        self.create_table(('pos INT', 'tree CHAR(20)'))
        c.executemany("INSERT INTO %s (pos,tree) VALUES (%%s,%%s)" % self.table,
                      list(enumerate('ash birch cedar Lärche pine'.split())))
        db.commit()

        c.execute("""
        CREATE PROCEDURE test_sp(IN t VARCHAR(255))
        BEGIN
            SELECT pos FROM %s WHERE tree = t;
        END
        """ % self.table)
        db.commit()

        c.callproc('test_sp', ('Lärche',))
        rows = c.fetchall()
        self.assertEqual(len(rows), 1)
        self.assertEqual(rows[0][0], 3)
        c.nextset()

        c.execute("DROP PROCEDURE test_sp")
        c.execute('drop table %s' % (self.table))

    def test_small_CHAR(self):
        # Character data
        def generator(row,col):
            i = (row*col+62)%256
            if i == 62: return ''
            if i == 63: return None
            return chr(i)
        self.check_data_integrity(
            ('col1 char(1)','col2 char(1)'),
            generator)

    def test_BIT(self):
        c = self.cursor
        try:
            c.execute("""create table test_BIT (
                b3 BIT(3),
                b7 BIT(10),
                b64 BIT(64))""")

            one64 = '1'*64
            c.execute(
                "insert into test_BIT (b3, b7, b64)"
                " VALUES (b'011', b'1111111111', b'%s')"
                % one64)

            c.execute("SELECT b3, b7, b64 FROM test_BIT")
            row = c.fetchone()
            self.assertEqual(row[0], b'\x03')
            self.assertEqual(row[1], b'\x03\xff')
            self.assertEqual(row[2], b'\xff'*8)
        finally:
            c.execute("drop table if exists test_BIT")

    def test_MULTIPOLYGON(self):
        c = self.cursor
        try:
            c.execute("""create table test_MULTIPOLYGON (
                id INTEGER PRIMARY KEY,
                border MULTIPOLYGON)""")

            c.execute(
                "insert into test_MULTIPOLYGON (id, border)"
                " VALUES (1, GeomFromText('MULTIPOLYGON(((1 1, 1 -1, -1 -1, -1 1, 1 1)),((1 1, 3 1, 3 3, 1 3, 1 1)))'))"
            )

            c.execute("SELECT id, AsText(border) FROM test_MULTIPOLYGON")
            row = c.fetchone()
            self.assertEqual(row[0], 1)
            self.assertEqual(row[1], 'MULTIPOLYGON(((1 1,1 -1,-1 -1,-1 1,1 1)),((1 1,3 1,3 3,1 3,1 1)))')

            c.execute("SELECT id, AsWKB(border) FROM test_MULTIPOLYGON")
            row = c.fetchone()
            self.assertEqual(row[0], 1)
            self.assertNotEqual(len(row[1]), 0)

            c.execute("SELECT id, border FROM test_MULTIPOLYGON")
            row = c.fetchone()
            self.assertEqual(row[0], 1)
            self.assertNotEqual(len(row[1]), 0)
        finally:
            c.execute("drop table if exists test_MULTIPOLYGON")

    def test_bug_2671682(self):
        from MySQLdb.constants import ER
        try:
            self.cursor.execute("describe some_non_existent_table");
        except self.connection.ProgrammingError as msg:
            self.assertTrue(str(ER.NO_SUCH_TABLE) in str(msg))

    def test_bug_3514287(self):
        c = self.cursor
        try:
            c.execute("""create table bug_3541287 (
                c1 CHAR(10),
                t1 TIMESTAMP)""")
            c.execute("insert into bug_3541287 (c1,t1) values (%s, NOW())",
                ("blah",))
        finally:
            c.execute("drop table if exists bug_3541287")

    def test_ping(self):
        self.connection.ping()

    def test_reraise_exception(self):
        c = self.cursor
        try:
            c.execute("SELECT x FROM not_existing_table")
        except MySQLdb.ProgrammingError as e:
            self.assertEqual(e.args[0], 1146)
            return
        self.fail("Should raise ProgrammingError")

    def test_binary_prefix(self):
        # verify prefix behaviour when enabled, disabled and for default (disabled)
        for binary_prefix in (True, False, None):
            kwargs = self.connect_kwargs.copy()
            # needs to be set to can guarantee CHARSET response for normal strings
            kwargs['charset'] = 'utf8'
            if binary_prefix != None:
                kwargs['binary_prefix'] = binary_prefix

            with closing(connection_factory(**kwargs)) as conn:
                with closing(conn.cursor()) as c:
                    c.execute('SELECT CHARSET(%s)', (MySQLdb.Binary(b'raw bytes'),))
                    self.assertEqual(c.fetchall()[0][0], 'binary' if binary_prefix else 'utf8')
                    # normal strings should not get prefix
                    c.execute('SELECT CHARSET(%s)', ('str',))
                    self.assertEqual(c.fetchall()[0][0], 'utf8')


if __name__ == '__main__':
    if test_MySQLdb.leak_test:
        import gc
        gc.enable()
        gc.set_debug(gc.DEBUG_LEAK)
    unittest.main()



#!/usr/bin/env python

import os
import io

import setuptools

if os.name == "posix":
    from setup_posix import get_config
else:  # assume windows
    from setup_windows import get_config

with io.open('README.md', encoding='utf-8') as f:
    readme = f.read()

metadata, options = get_config()
metadata['ext_modules'] = [
    setuptools.Extension("MySQLdb._mysql", sources=['MySQLdb/_mysql.c'], **options)
]
metadata['long_description'] = readme
metadata['long_description_content_type'] = "text/markdown"
setuptools.setup(**metadata)



import os, sys
try:
    from ConfigParser import SafeConfigParser
except ImportError:
    from configparser import ConfigParser as SafeConfigParser

# This dequote() business is required for some older versions
# of mysql_config

def dequote(s):
    if not s:
        raise Exception("Wrong MySQL configuration: maybe https://bugs.mysql.com/bug.php?id=86971 ?")
    if s[0] in "\"'" and s[0] == s[-1]:
        s = s[1:-1]
    return s

_mysql_config_path = "mysql_config"

def mysql_config(what):
    from os import popen

    f = popen("%s --%s" % (_mysql_config_path, what))
    data = f.read().strip().split()
    ret = f.close()
    if ret:
        if ret/256:
            data = []
        if ret/256 > 1:
            raise EnvironmentError("%s not found" % (_mysql_config_path,))
    return data

def get_config():
    from setup_common import get_metadata_and_options, enabled, create_release_file
    global _mysql_config_path

    metadata, options = get_metadata_and_options()

    if 'mysql_config' in options:
        _mysql_config_path = options['mysql_config']
    else:
        try:
            mysql_config('version')
        except EnvironmentError:
            # try mariadb_config
            _mysql_config_path = "mariadb_config"
            try:
                mysql_config('version')
            except EnvironmentError:
                _mysql_config_path = "mysql_config"

    extra_objects = []
    static = enabled(options, 'static')

    # allow a command-line option to override the base config file to permit
    # a static build to be created via requirements.txt
    #
    if '--static' in sys.argv:
        static = True
        sys.argv.remove('--static')

    libs = mysql_config("libs")
    library_dirs = [dequote(i[2:]) for i in libs if i.startswith('-L')]
    libraries = [dequote(i[2:]) for i in libs if i.startswith('-l')]
    extra_link_args = [x for x in libs if not x.startswith(('-l', '-L'))]

    removable_compile_args = ('-I', '-L', '-l')
    extra_compile_args = [i.replace("%", "%%") for i in mysql_config("cflags")
                          if i[:2] not in removable_compile_args]

    # Copy the arch flags for linking as well
    for i in range(len(extra_compile_args)):
        if extra_compile_args[i] == '-arch':
            extra_link_args += ['-arch', extra_compile_args[i + 1]]

    include_dirs = [dequote(i[2:])
                    for i in mysql_config('include') if i.startswith('-I')]

    if static:
        # properly handle mysql client libraries that are not called libmysqlclient
        client = None
        CLIENT_LIST = ['mysqlclient', 'mysqlclient_r', 'mysqld', 'mariadb',
                       'mariadbclient', 'perconaserverclient', 'perconaserverclient_r']
        for c in CLIENT_LIST:
            if c in libraries:
                client = c
                break

        if client == 'mariadb':
            client = 'mariadbclient'
        if client is None:
            raise ValueError("Couldn't identify mysql client library")

        extra_objects.append(os.path.join(library_dirs[0], 'lib%s.a' % client))
        if client in libraries:
            libraries.remove(client)

    name = "mysqlclient"
    metadata['name'] = name

    define_macros = [
        ('version_info', metadata['version_info']),
        ('__version__', metadata['version']),
        ]
    create_release_file(metadata)
    del metadata['version_info']
    ext_options = dict(
        library_dirs = library_dirs,
        libraries = libraries,
        extra_compile_args = extra_compile_args,
        extra_link_args = extra_link_args,
        include_dirs = include_dirs,
        extra_objects = extra_objects,
        define_macros = define_macros,
    )

    # newer versions of gcc require libstdc++ if doing a static build
    if static:
        ext_options['language'] = 'c++'

    return metadata, ext_options

if __name__ == "__main__":
    sys.stderr.write("""You shouldn't be running this directly; it is used by setup.py.""")
